<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00022000CalculoFormacaoPrecoFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * A função "recebeOrdenacao" recebe a ordenação máxima para cálculo.
     */
     
    public function recebeOrdenacao($params){
        $sql = "SELECT NVL(MAX(IDORDENACAO), 0) AS IDORDENACAO
                  FROM FCT_CALCFORMPRECSERV
                 WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                   AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "recebeOrdenacaoAux" recebe a ordenação máxima para cálculo
     * da tabela de formações de preço alteradas durante a inserção da proposta.
     */
     
    public function recebeOrdenacaoAux($params){
        $sql = "SELECT NVL(MAX(IDORDENACAO), 0) AS IDORDENACAO
                FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                LEFT JOIN FCT_ORCAMENTO ORC
                    ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND CALC.NRORG         = ORC.NRORG
                WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                        OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                        OR (NRITEMORCAMENTO IN(-1,0) 
                                AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                            ))
                    AND CALC.NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função "recebeOrdenacaoAuxCont" recebe a ordenação máxima para cálculo
     * da tabela de formações de preço alteradas durante a inserção do contrato.
     */
     
    public function recebeOrdenacaoAuxCont($params){
        $sql = "SELECT NVL(MAX(IDORDENACAO), 0) AS IDORDENACAO
                  FROM FCT_CALCFORMPRECSERV_ITCONT
                 WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
                    AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
  
    /**
     *  A função "validaIdentificador" Realiza a validação  do campo "identificador". 
     *  Não podem haver dois identificadores iguais em uma mesma composição.
     */
     
    public function validaIdentificador($params){
        
        $sql = "SELECT COUNT(*) AS VALIDA 
                  FROM FCT_CALCFORMPRECSERV 
                 WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                   AND IDITEMCALCULO = :IDITEMCALCULO
                   AND NRCALCFORMPRECSERV <> NVL(:NRCALCFORMPRECSERV, -1)
                   AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     *  A função "validaIdentificadorAux" Realiza a validação  do campo "identificador" das 
     * formações de preço alteradas na proposta durante a inserção dos itens
     * caso a proposta seja do tipo alimenticia ele ira testar o número do relacionamento 
     * entre a proposta e a formação ao invés do nrdoitem
     */
    public function validaIdentificadorAux($params){
        $sql = "SELECT COUNT(*) AS VALIDA 
                  FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                LEFT JOIN FCT_ORCAMENTO ORC
                    ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND CALC.NRORG         = ORC.NRORG
                WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND IDITEMCALCULO = :IDITEMCALCULO
                    AND NRCALCFORMPRECSERV <> NVL(:NRCALCFORMPRECSERV, -1)
                    AND ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                        OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                        OR (NRITEMORCAMENTO IN(-1,0) 
                                AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                            ))
                    AND CALC.NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     *  A função "validaIdentificadorAuxCont" Realiza a validação  do campo "identificador" das 
     * formações de preço alteradas no contrato durante a inserção dos itens
     */
    public function validaIdentificadorAuxCont($params){
        $sql = "SELECT COUNT(*) AS VALIDA 
                  FROM FCT_CALCFORMPRECSERV_ITCONT 
                WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND IDITEMCALCULO = :IDITEMCALCULO
                    AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
                    AND NRCALCFORMPRECSERV <> NVL(:NRCALCFORMPRECSERV, -1)
                    AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     *  A função "buscaIdentValores" busca os identificadores dos elementos e seus respectivos valores
     */
    public function buscaIdentValores($params){
        $sql = "SELECT IDELEMENTO, IDITEMCALCULO, IDTIPOCALCULO, NRCALCFORMPRECSERV,
                       DSEXPRESSAOCALC, VRTOTAL, VRPERCCALCULO, VRUNITARIO, QTCALCULO, 
                       IDUSAMEDIDA, QTMEDIDACALC
                FROM FCT_CALCFORMPRECSERV 
                WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                   AND IDITEMCALCULO = :IDITEMCALCULO
                   AND NRORG = :NRORG
                ORDER BY IDORDENACAO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     *  A função "buscaIdentValores" busca os identificadores dos elementos e seus respectivos valores
     */
    public function buscaIdentValoresAux($params){
        $sql = "SELECT IDELEMENTO, IDITEMCALCULO, IDTIPOCALCULO, NRCALCFORMPRECSERV,
                       DSEXPRESSAOCALC, VRTOTAL, VRPERCCALCULO, VRUNITARIO, QTCALCULO, 
                       IDUSAMEDIDA, QTMEDIDACALC
                FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                LEFT JOIN FCT_ORCAMENTO ORC
                    ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND CALC.NRORG         = ORC.NRORG
                WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND IDITEMCALCULO    = :IDITEMCALCULO
                    AND ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                        OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                        OR (NRITEMORCAMENTO IN(-1,0)  
                                AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                            ))
                    AND CALC.NRORG = :NRORG
                ORDER BY IDORDENACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     *  A função "buscaIdentValores" busca os identificadores dos elementos e seus respectivos valores
     */
    public function buscaIdentValoresCont($params){
        $sql = "SELECT IDELEMENTO, IDITEMCALCULO, IDTIPOCALCULO, NRCALCFORMPRECSERV,
                       DSEXPRESSAOCALC, VRTOTAL, VRPERCCALCULO, VRUNITARIO, QTCALCULO, 
                       IDUSAMEDIDA, QTMEDIDACALC
                FROM FCT_CALCFORMPRECSERV_ITCONT 
                WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                    AND IDITEMCALCULO = :IDITEMCALCULO
                    AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
                    AND NRORG = :NRORG
                ORDER BY IDORDENACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function buscaIdentValoresAnte($params){
        $sql = "SELECT IDELEMENTO, IDITEMCALCULO, VRTOTAL 
                  FROM FCT_CALCFORMPRECSERV 
                 WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                   AND NRORG = :NRORG
                ORDER BY IDORDENACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     *  A função "calcSomatorioFilhos" calcula o somatorio dos elementos filhos 
     */
    public function calcSomatorioFilhos($params){
        $sql = "SELECT SUM(VRTOTAL) AS VRSOMA, SUM(VRPERCCALCULO) AS VRPERCENTUAL
                  FROM FCT_CALCFORMPRECSERV
                 WHERE NRORG = :NRORG
                   AND NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERV";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     *  A função "calcSomatorioFilhos" calcula o somatorio dos elementos filhos 
     * das formações criadas na proposta
     */
    public function calcSomatorioFilhos_ItPro($params){
        $sql = "SELECT SUM(VRTOTAL) AS VRSOMA, SUM(VRPERCCALCULO) AS VRPERCENTUAL
                FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                LEFT JOIN FCT_ORCAMENTO ORC
                    ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND CALC.NRORG         = ORC.NRORG
                WHERE NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERV
                    AND ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                        OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                        OR (NRITEMORCAMENTO IN(-1,0)  
                                AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                            ))
                    AND CALC.NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     *  A função "calcSomatorioFilhos" calcula o somatorio dos elementos filhos 
     * das formações criadas no contrato
     */
    public function calcSomatorioFilhos_ItCont($params){
        $sql = "SELECT SUM(VRTOTAL) AS VRSOMA, SUM(VRPERCCALCULO) AS VRPERCENTUAL
                  FROM FCT_CALCFORMPRECSERV_ITCONT
                 WHERE NRORG = :NRORG
                   AND NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERV
                   AND (NRITEMOCONTRATO = :NRITEMOCONTRATO 
                        OR (:NRITEMOCONTRATO IS NULL AND NRITEMOCONTRATO IS NULL))";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     *  A função "updSomatorioFilhos" atualiza o elemento com o somatiorio dos filhos
     */
    public function updSomatorioFilhos($params){
        $sql = "UPDATE FCT_CALCFORMPRECSERV
                   SET VRUNITARIO = :VRSOMA,
                       VRTOTAL = :VRSOMA * QTCALCULO
                 WHERE NRORG = :NRORG
                   AND NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
     /**
     *  A função "deletarPaieFilhos"  deleta os pais e os filhos.
     */
    public function deletarPaieFilhos($params){
        $sql = "DELETE FROM FCT_CALCFORMPRECSERV
                 WHERE NRORG      = :NRORG
                   AND NRCALCFORMPRECSERV IN (SELECT NRCALCFORMPRECSERV
                                                FROM FCT_CALCFORMPRECSERV
                                               WHERE NRORG  = :NRORG
                                               START WITH NRCALCFORMPRECSERV  = :NRCALCFORMPRECSERV
                                             CONNECT BY PRIOR NRCALCFORMPRECSERV = NRCALCFORMPRECSERVPAI)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /* A função "getElementoCalcFormPrec" busca os elementos para realização do recálculo */
    public function getElementoCalcFormPrec($params){
        $sql = "SELECT IDITEMCALCULO, NRCALCFORMPRECSERV, DSREFECALCULO, NRFORMPRECOSERVICO, IDTIPOCALCULO,
                       DSEXPRESSAOCALC, VRPERCCALCULO, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, QTMEDIDACALC, VRTOTAL
                FROM FCT_CALCFORMPRECSERV
                WHERE NRORG  = :NRORG
                   AND IDTIPOCALCULO <> 'V'
                   AND NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função busca os elementos  alterados na proposta 
     * para realização do recálculo
    **/
    public function getElementoCalcFormPrecProp($params){
        $sql = "SELECT IDITEMCALCULO, NRCALCFORMPRECSERV, DSREFECALCULO, NRFORMPRECOSERVICO, IDTIPOCALCULO,
                       DSEXPRESSAOCALC, VRPERCCALCULO, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, QTMEDIDACALC, VRTOTAL
                FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                LEFT JOIN FCT_ORCAMENTO ORC
                    ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND CALC.NRORG         = ORC.NRORG
                WHERE IDTIPOCALCULO <> 'V'
                    AND ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                        OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                        OR (NRITEMORCAMENTO IN(-1,0)  
                                AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                            ))
                    AND CALC.NRORG = :NRORG
                    AND NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função busca os elementos  alterados no contrato 
     * para realização do recálculo
    **/
    public function getElementoCalcFormPrecCont($params){
        $sql = "SELECT IDITEMCALCULO, NRCALCFORMPRECSERV, DSREFECALCULO, NRFORMPRECOSERVICO, IDTIPOCALCULO,
                       DSEXPRESSAOCALC, VRPERCCALCULO, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, QTMEDIDACALC, VRTOTAL
                  FROM FCT_CALCFORMPRECSERV_ITCONT
                 WHERE NRORG  = :NRORG
                   AND IDTIPOCALCULO <> 'V'
                   AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
                   AND NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* A função "getElementoCalcFormPrec" busca os elementos filhos para realização do recálculo */
    public function getElementosFilhosCalcFormPrec($params){
        $sql = "SELECT IDITEMCALCULO, NRCALCFORMPRECSERV, DSREFECALCULO, NRFORMPRECOSERVICO, IDTIPOCALCULO,
                       DSEXPRESSAOCALC, VRPERCCALCULO, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, QTMEDIDACALC, VRTOTAL
                FROM FCT_CALCFORMPRECSERV
                WHERE NRORG  = :NRORG
                   AND NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERV
                ORDER BY IDTIPOCALCULO DESC, NRCALCFORMPRECSERV ASC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função busca os elementos filhos alterados na proposta
     * para realização do recálculo 
    **/
    
    public function getElementosFilhosCalcFormPrecProp($params){
        $sql = "SELECT IDITEMCALCULO, NRCALCFORMPRECSERV, DSREFECALCULO, NRFORMPRECOSERVICO, IDTIPOCALCULO,
                       DSEXPRESSAOCALC, VRPERCCALCULO, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, QTMEDIDACALC, VRTOTAL
                FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                LEFT JOIN FCT_ORCAMENTO ORC
                    ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND CALC.NRORG         = ORC.NRORG
                WHERE ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                        OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                        OR (NRITEMORCAMENTO IN(-1,0)  
                                AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                            ))
                    AND CALC.NRORG = :NRORG
                    AND NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERV
                ORDER BY IDTIPOCALCULO DESC, NRCALCFORMPRECSERV ASC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * A função busca os elementos filhos alterados na proposta
     * para realização do recálculo 
    **/
    
    public function getElementosFilhosCalcFormPrecCont($params){
        $sql = "SELECT IDITEMCALCULO, NRCALCFORMPRECSERV, DSREFECALCULO, NRFORMPRECOSERVICO, IDTIPOCALCULO,
                       DSEXPRESSAOCALC, VRPERCCALCULO, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, QTMEDIDACALC, VRTOTAL
                FROM FCT_CALCFORMPRECSERV_ITCONT
                WHERE NRORG  = :NRORG
                   AND NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERV
                   AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
                ORDER BY IDTIPOCALCULO DESC, NRCALCFORMPRECSERV ASC";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

     /**
     *  A função "salvarRecalculo" salva o recalculo dos elementos.
     */
     
    public function salvarRecalculo($params){
        $sql = "UPDATE FCT_CALCFORMPRECSERV
                   SET VRUNITARIO = :VRUNITARIO,
                   QTCALCULO      = :QTCALCULO,
                   QTMEDIDACALC   = :QTMEDIDACALC,
                   VRTOTAL        = :VRTOTAL,
                   VRPERCCALCULO  = :VRPERCCALCULO
                WHERE NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV
                   AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
     /**
     *  A função salva o recalculo dos elementos alterados na proposta.
     **/
     
    public function salvarRecalculoProp($params){
        $sql = "UPDATE FCT_CALCFORMPRECSERV_ITPRO
                   SET VRUNITARIO = :VRUNITARIO,
                   QTCALCULO      = :QTCALCULO,
                   QTMEDIDACALC   = :QTMEDIDACALC,
                   VRTOTAL        = :VRTOTAL,
                   VRPERCCALCULO  = :VRPERCCALCULO
                WHERE NRCALCFORMPRECSERV = (SELECT NRCALCFORMPRECSERV 
                                            FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                                            LEFT JOIN FCT_ORCAMENTO ORC
                                                ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                                                AND CALC.NRORG         = ORC.NRORG
                                            WHERE ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                                                    OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                                                    OR (NRITEMORCAMENTO IN(-1,0)  
                                                            AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                                            AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                                                        ))
                                                AND CALC.NRORG = :NRORG
                                                AND NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
     /**
     *  A função salva o recalculo dos elementos alterados no contrato.
     **/
    public function salvarRecalculoCont($params){
        $sql = "UPDATE FCT_CALCFORMPRECSERV_ITCONT
                   SET VRUNITARIO = :VRUNITARIO,
                   QTCALCULO      = :QTCALCULO,
                   QTMEDIDACALC   = :QTMEDIDACALC,
                   VRTOTAL        = :VRTOTAL,
                   VRPERCCALCULO  = :VRPERCCALCULO
                WHERE NRCALCFORMPRECSERV = :NRCALCFORMPRECSERV
                    AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
                    AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
         /**
     *  A função "salvarRecalculo" salva o recalculo dos elementos.
     */
     
    public function insereItensCadAuto($params){
        $sql = "INSERT INTO FCT_CALCFORMPRECSERV (
                    NRCALCFORMPRECSERV, NRFORMPRECOSERVICO, IDORDENACAO, IDITEMCALCULO, IDELEMENTO, NRCALCFORMPRECSERVPAI,
                    DSREFECALCULO, IDTIPOCALCULO, DSEXPRESSAOCALC, VRUNITARIO, QTCALCULO, IDUSAMEDIDA, 
                    QTMEDIDACALC, VRTOTAL, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, 
                    CDOPERINCLUSAO, CDOPERULTATU, VRPERCCALCULO
                ) VALUES (
                    :NRCALCFORMPRECSERV, :NRFORMPRECOSERVICO, :IDORDENACAO, :IDITEMCALCULO, :IDELEMENTO, :NRCALCFORMPRECSERVPAI,
                    :DSREFECALCULO, :IDTIPOCALCULO, :DSEXPRESSAOCALC, 0, 1, 'N', 
                    NULL, 0, :NRORG, 'S', SYSDATE, :NRORG, 
                    :CDOPERADOR, :CDOPERULTATU, :VRPERCCALCULO
                )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
     /**
     *  A função "atualizaItensCadAuto" cria a hierarquização dos itens.
     */
     
    public function atualizaItensCadAuto($params){
        $sql = "UPDATE FCT_CALCFORMPRECSERV
                   SET NRCALCFORMPRECSERVPAI = :NRCALCFORMPRECSERVPAI
                 WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                   AND CDOPERULTATU       = :CDOPERULTATU
                   AND NRORG              = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
     /**
     *  A função "limpaExpressao" limpa o campo de expressão utilizado no cadastramento automatico.
     */
     
    public function limpaExpressao($params){
        $sql = "UPDATE FCT_CALCFORMPRECSERV
                   SET CDOPERULTATU = NULL
                 WHERE NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                   AND CDOPERULTATU LIKE 'CAD:%'
                   AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
    /* A função "getElementosArvoreCalcFormPrev" busca os elementos para realização do recálculo da árvore. */
    
    public function getElementosArvoreCalcFormPrev(array $params) {
        $sql = "SELECT LEVEL,
                  CALC.IDITEMCALCULO,
                  CALC.NRCALCFORMPRECSERV,
                  CALC.DSREFECALCULO,
                  CALC.NRFORMPRECOSERVICO,
                  CALC.IDTIPOCALCULO,
                  CALC.DSEXPRESSAOCALC,
                  CALC.VRPERCCALCULO,
                  CALC.VRUNITARIO,
                  CALC.QTCALCULO,
                  CALC.IDUSAMEDIDA,
                  CALC.QTMEDIDACALC
                  FROM FCT_CALCFORMPRECSERV CALC
                 WHERE CALC.NRORG                       = :NRORG
                   AND CALC.IDTIPOCALCULO               <> 'V'
                   AND CALC.NRFORMPRECOSERVICO          = :NRFORMPRECOSERVICO
               CONNECT BY PRIOR CALC.NRCALCFORMPRECSERV = CALC.NRCALCFORMPRECSERVPAI
                 START WITH CALC.NRCALCFORMPRECSERV     IN (SELECT NRCALCFORMPRECSERV
                                                              FROM FCT_CALCFORMPRECSERV
                                                             WHERE NRCALCFORMPRECSERVPAI IS NULL
                                                               AND NRFORMPRECOSERVICO    = CALC.NRFORMPRECOSERVICO
                                                               AND NRORG                 = CALC.NRORG)
                 ORDER BY LEVEL DESC, CALC.IDTIPOCALCULO DESC, CALC.NRCALCFORMPRECSERV";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
    * A função busca os elementos alterados durante a inserção da proposta
    * para realização do recálculo da árvore.
    **/
    
    public function getElementosArvoreCalcFormProp(array $params) {
        $sql = "SELECT LEVEL,
                  CALC.IDITEMCALCULO,
                  CALC.NRCALCFORMPRECSERV,
                  CALC.DSREFECALCULO,
                  CALC.NRFORMPRECOSERVICO,
                  CALC.IDTIPOCALCULO,
                  CALC.DSEXPRESSAOCALC,
                  CALC.VRPERCCALCULO,
                  CALC.VRUNITARIO,
                  CALC.QTCALCULO,
                  CALC.IDUSAMEDIDA,
                  CALC.QTMEDIDACALC
                  FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                 WHERE CALC.NRORG                       = :NRORG
                   AND CALC.IDTIPOCALCULO               <> 'V'
                   AND CALC.NRFORMPRECOSERVICO          = :NRFORMPRECOSERVICO
               CONNECT BY PRIOR CALC.NRCALCFORMPRECSERV = CALC.NRCALCFORMPRECSERVPAI
                 START WITH CALC.NRCALCFORMPRECSERV IN (SELECT NRCALCFORMPRECSERV
                                                            FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                                                            LEFT JOIN FCT_ORCAMENTO ORC
                                                                ON CALC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                                                                AND CALC.NRORG         = ORC.NRORG
                                                            WHERE ((:NRITEMORCAMENTO  IS NULL AND NRITEMORCAMENTO IS NULL)
                                                                    OR (NRITEMORCAMENTO = :NRITEMORCAMENTO)
                                                                    OR (NRITEMORCAMENTO IN(-1,0) 
                                                                            AND NRRELACFORMACAOPROP = :NRITEMORCAMENTO
                                                                            AND (ORC.IDPROPALIMENTICIA = 'S' OR ORC.NRSEQORCAMENTO IS NULL)
                                                                        ))
                                                                AND NRCALCFORMPRECSERVPAI IS NULL
                                                                AND CALC.NRORG = :NRORG)
                 ORDER BY LEVEL DESC, CALC.IDTIPOCALCULO DESC, CALC.NRCALCFORMPRECSERV";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
    * A função busca os elementos alterados durante a inserção do contrato
    * para realização do recálculo da árvore.
    **/
    public function getElementosArvoreCalcFormCont(array $params) {
        $sql = "SELECT LEVEL,
                  CALC.IDITEMCALCULO,
                  CALC.NRCALCFORMPRECSERV,
                  CALC.DSREFECALCULO,
                  CALC.NRFORMPRECOSERVICO,
                  CALC.IDTIPOCALCULO,
                  CALC.DSEXPRESSAOCALC,
                  CALC.VRPERCCALCULO,
                  CALC.VRUNITARIO,
                  CALC.QTCALCULO,
                  CALC.IDUSAMEDIDA,
                  CALC.QTMEDIDACALC
                  FROM FCT_CALCFORMPRECSERV_ITCONT CALC
                 WHERE CALC.NRORG                       = :NRORG
                   AND CALC.IDTIPOCALCULO               <> 'V'
                   AND CALC.NRFORMPRECOSERVICO          = :NRFORMPRECOSERVICO
                   AND ((:NRITEMOCONTRATO  IS NULL AND NRITEMOCONTRATO IS NULL)
                        OR (NRITEMOCONTRATO = :NRITEMOCONTRATO))
               CONNECT BY PRIOR CALC.NRCALCFORMPRECSERV = CALC.NRCALCFORMPRECSERVPAI
                 START WITH CALC.NRCALCFORMPRECSERV     IN (SELECT NRCALCFORMPRECSERV
                                                              FROM FCT_CALCFORMPRECSERV_ITCONT
                                                             WHERE NRCALCFORMPRECSERVPAI IS NULL
                                                               AND NRFORMPRECOSERVICO    = CALC.NRFORMPRECOSERVICO
                                                               AND NRORG                 = CALC.NRORG)
                 ORDER BY LEVEL DESC, CALC.IDTIPOCALCULO DESC, CALC.NRCALCFORMPRECSERV";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}
?>