<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\IntegracaoFmFactory;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Session\Session;
use Teknisa\SM\Service\IntegracoesTeknisaService;

class IntegracaoFmService {

    protected $integracaoFactory;
    protected $environment;
    protected $newCode;
    protected $session;
    protected $connection;
    protected $integracoesTeknisaService;

    public function __construct(IntegracaoFmFactory $integracaoFactory, Environment $environment, Session $session, IntegracoesTeknisaService $integracoesTeknisaService) {
        $this->integracaoFactory         = $integracaoFactory;
        $this->environment               = $environment;
        $this->session                   = $session;
        $this->integracoesTeknisaService = $integracoesTeknisaService;
    }

    /**
     * Retorna a URL de integração com o FM.
     * 
     * @param integer $nrorg
     * 
     * @return string
     */
    public function getUrlIntegracaoFM($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getUrlIntegracaoFM($params);
        return $dataset[0]["DSURLINTEGRACAOFM"];
    }
  
    /**
     * Verifica se o contrato utiliza a integração com o FM.
     * 
     * @param integer $nrseqcontrato
     * @param integer $nrorg
     * 
     * @return mixed[]
     */
    public function verificarUtilizaIntegracao($nrseqcontrato, $nrorg) {

        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->verificarUtilizaIntegracao($params);

        $utilizaInt = array(
            "UTILIZA" => $dataset[0]["IDINTEGRACAOFM"],
            "LOCAIS"  => $dataset[0]["IDINTEGRALOCAISFM"],
            "MEDICAO" => $dataset[0]["IDINTEGRAMEDICAOFM"]
        );

        return $utilizaInt;
    }

    /**
     * Retorna o tipo de local para integração dos locais de atendimento.
     * 
     * @param integer $nrorg
     * 
     * @return string
     */
    public function getTipoLocalIntegracaoFM($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getTipoLocalIntegracaoFM($params);
        return $dataset[0]["NRTIPOLOCALCLIFM"];
    }

    /**
     * Retorna os dados de local para integração dos locais de atendimento.
     * 
     * @param mixed[] $params
     * 
     * @return mixed[]
     */
    public function getDadosLocais($nrseqcontrato, $nrorg) {

        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getDadosLocais($params);
        return $dataset;
    }

    public function enviarLocaisIntegracaoFM($nrseqcontrato, $nrorg, $operador, $tipoLocalIntegracao, $URLIntegracaoFM) {
        try{

            $locais = $this->getDadosLocais($nrseqcontrato, $nrorg);

            if (count($locais) != 0){ 

                $dadosAux = array();

                foreach($locais as $ind => $local){

                    $dadosAux[$ind] = array(
                        "NRESTRUTURAM"   => $local["NRESTRUTURAM"], 
                        "NMESTRUTURAM"   => $local["NMESTRUTURAM"], 
                        "CDCLIENTE"      => $local["CDCLIENTE"],
                        "NRTIPOLOCALCLI" => $tipoLocalIntegracao,
                        "NRORG"          => $nrorg,
                        "CDOPERADOR"     => $operador
                    );
                }

                $dados = array(
                    "requestType" => "DataSet",
                    "dataset"     => array($dadosAux)
                );

                $retorno = $this->integracoesTeknisaService->executarCurl(null, $URLIntegracaoFM."/IntegracaoSM/cadastrarLocais", $dados);
                
            }
            else {

                $retorno = 'OK';
            }

            return $retorno;
        }
        catch (\Exception $e) {

            return $e->getMessage();
        }
    }

    public function getMedicaoIntegracaoFM($nrseqcontrato, $nrmedicaofatura, $nrorg, $operador, $URLIntegracaoFM) {
        try{
            $dadosAux = array(
                "NRSEQCONTRATO"   => $nrseqcontrato, 
                "NRMEDICAOFATURA" => $nrmedicaofatura, 
                "NRORG"           => $nrorg,
                "CDOPERADOR"      => $operador
            );

            $dados = array(
                "requestType" => "DataSet",
                "dataset"     => array($dadosAux)
            );

            $retorno = $this->integracoesTeknisaService->executarCurl(null, $URLIntegracaoFM."/IntegracaoSM/getDadosMedicao", $dados);
            
            if ($retorno["RETORNO"] != 'OK') {

                $retorno = $retorno["RETORNO"];
            }
            else{

                foreach($retorno["DADOS"] as $medicao){
                    $params = array(
                        "QTMEDIDAVARIAVEL"    => $medicao["QTMEDIDA"],
                        "NRORG"               => $nrorg,
                        "OPERADOR"            => $operador,
                        "NRMEDICAOFATURA"     => $medicao["NRMEDICAOFATURA"],
                        "CDSERVORCA"          => $medicao["SERVICO"],
                        "CDTPMEDIDAVARIAVEL"  => $medicao["MEDIDA"]
                    );  

                    $this->integracaoFactory->updateDadosMedicao($params);

                }

                $retorno = 'OK';
            }

            return $retorno;
        }
        catch (\Exception $e) {
            return $e->getMessage();
        }
    }

    public function integraLocalAtend($nrseqcontrato) {
        $nrorg = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        
        try{
            $validaIntegracaoFM = $this->verificarUtilizaIntegracao($nrseqcontrato, $nrorg);
            $URLIntegracaoFM = $this->getUrlIntegracaoFM($nrorg);
            $tipoLocalIntegracao = $this->getTipoLocalIntegracaoFM($nrorg);

            if (($validaIntegracaoFM["UTILIZA"] == 'S') && ($validaIntegracaoFM["LOCAIS"] == 'S')) {
                if ($URLIntegracaoFM != null) {
                    if ($tipoLocalIntegracao != null) {
                        $retorno = $this->enviarLocaisIntegracaoFM($nrseqcontrato, $nrorg, $operador, $tipoLocalIntegracao, $URLIntegracaoFM);
                    }
                    else {
                        $retorno = 'Não foi definido o tipo de local para integração dos locais de atendimento do módulo "SM" para o módulo "FM". <br><br> Favor definir o parâmetro "Tipo de Local (Integração FM)" na tela "Parametrização SM".';
                    }
                }
                else {
                    $retorno = 'Não foi definida a URL para integração dos locais de atendimento do módulo "SM" para o módulo "FM". <br><br> Favor definir o parâmetro "Url Integração (FM)" na tela "Parametrização SM".';
                }
            }
            else {
                $retorno = 'OK';
            }

            return $retorno;
        }
        catch (\Exception $e) {
            return $e->getMessage();
        }
    }
    
    public function importarMedicaoFM($nrseqcontrato, $nrmedicaofatura) {
        $nrorg = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        
        try {
            $validaIntegracaoFM = $this->verificarUtilizaIntegracao($nrseqcontrato, $nrorg);
            $URLIntegracaoFM = $this->getUrlIntegracaoFM($nrorg);
            //$tipoLocalIntegracao = $this->getTipoLocalIntegracaoFM($nrorg);

            if (($validaIntegracaoFM["UTILIZA"] == 'S') && ($validaIntegracaoFM["MEDICAO"] == 'S')) {
                if ($URLIntegracaoFM != null) {
                    $retorno = $this->getMedicaoIntegracaoFM($nrseqcontrato, $nrmedicaofatura, $nrorg, $operador, $URLIntegracaoFM);
                }
                else {
                    $retorno = 'Não foi definida a URL para integração do módulo "SM" com o módulo "FM". <br><br> Favor definir o parâmetro "URL Integração (FM)" na tela "Parametrização SM".';
                }
            }
            else {
                $retorno = 'OK';
            }

            return $retorno;
        }
        catch (\Exception $e) {
            return $e->getMessage();
        }
    }
 
}

?>